# -*- coding:Utf-8 -*-
import es, psyco
psyco.full()

esInfo = es.AddonInfo()
esInfo.author = "DoCky"
esInfo.version = "2.003p"
esInfo.name = "PlayerViewLib"
esInfo.basename = "playerview"
esInfo.url = "http://forums.eventscripts.com/viewtopic.php?t=42292"
esInfo.description = "Allow you to know what entity a player is aiming"

lastgive = es.ServerVar("eventscripts_lastgive") # for "coord" function
version = es.ServerVar("playerviewlib", "", " Created by DoCky -")
version.set(esInfo.version)
version.makepublic()

_plUserid = es.ServerVar("playerview_userid", 0, "playerview: userid of player")
_plFunction = es.ServerVar("playerview_function", "", "playerview: name of function executed")
_plClassname = es.ServerVar("playerview_classname", "", "playerview: name of entity viewved (for entity, player & weapon commands)")
_plEntityid = es.ServerVar("playerview_entityid", 0, "playerview: index of entity aimed (-1 if none)")
_plPlayerid = es.ServerVar("playerview_playerid", 0, "playerview: userid of player aimed (0 if none)")
_plPlayerindex = es.ServerVar("playerview_playerindex", -1, "playerview: index of player aimed (-1 if none)")
_plWeaponid = es.ServerVar("playerview_weaponid", 0, "playerview: index of weapon aimed (-1 if none)")
_plCoordinates = es.ServerVar("playerview_coordinates", "0,0,0", "playerview: coordinates where the player is aiming")

_cvars = {
    "userid" : _plUserid, "function" : _plFunction, "classname" : _plClassname,
    "entityid" : _plEntityid, "playerid" : _plPlayerid, "playerindex" : _plPlayerindex, "weaponid" : _plWeaponid, "coordinates" : _plCoordinates}

# players dict
_players = {}
_weapon = "tf_weapon_" if es.getGameName() == "tf" else "weapon_"
_functions = "entity", "player", "weapon", "coord"

def _command():
    function = es.getargv(2)
    # check if function exists
    if not function in _functions: raise ValueError, 'playerview: "%s" is not a valid function' % function
    # exec internal i_coord method
    if function == "coord": _players[int(es.getargv(1))].i_coord(es.getargv(3))
    # exec internal i_entity method
    else: _players[int(es.getargv(1))].i_entity(function)
# register block for internal command
es.addons.registerBlock("_playerviewlib", "_command", _command)
# register server command
es.regcmd("_playerview_cmd", "_playerviewlib/_command", "playerview: internal command")

class PlayerView(object):
    """ PlayerView object """
    def __init__(self, userid, callback = None):
        # internal attribute
        self._entities = {}
        self._entityname = ""

        # store userid, callback function and block
        self.userid = int(userid)
        self.callback = callback
        self.block = ""

        # public attributes
        self.parameters = None
        self.function = ""
        self.classname = ""

        self.entityid = -1
        self.playerid = 0
        self.playerindex = -1
        self.weaponid = -1
        self.coordinates = "0,0,0"

    def __setitem__(self, cvar, value):
        if _cvars.has_key(cvar): _cvars[cvar].set(value)

    def __getattr__(self, name):
        if name == "coordlist": return map(float, self.coordinates.split(","))

    def i_command(self, refresh = True):
        if not es.exists("userid", self.userid): raise ValueError, 'playerview: userid "%s" does not exists' % self.userid
        if not _players.has_key(self.userid):
            # refresh entity list
            if refresh: self.i_entities()
            # add object in players dict
            _players[self.userid] = self

    def i_entities(self):
        # clear dict
        self._entities.clear()
        # for entities in map
        for index, attribute in es.createentityindexlist().items():
            # store "index" = "classname", "targetname"
            self._entities[index] = attribute["classname"], es.entitygetvalue(index, "targetname")


    """ entity, player & weapon functions """
    def i_entity(self, function):
        # default classname value
        self.classname = ""
        # set function name
        self.function = function

        # default index value
        entity = -1
        # search status
        found = False
        # targetname to find
        targetname = "_playerview_%s_%s" % (function, self.userid)
        # for index, info in entity list
        for index in self._entities:
            # check if targetname is corresponding
            if es.entitygetvalue(index, "targetname") == targetname:
                found = True
                entity = index
                self.classname, targetname = self._entities[index]
                # restore last targetname
                es.entitysetvalue(index, "targetname", targetname)
                break
        if not found: entity = -1

        # remove player from dict
        del _players[self.userid]
        # set basic convars
        if self.block is not None:
            self["userid"] = self.userid
            self["function"] = function
            self["classname"] = ""

        # if function is "entity"
        if function == "entity":
            if self.classname.startswith(self._entityname):
                self.entityid = entity
                if self.block is not None: self["classname"] = self.classname
            else: self.entityid, self.classname = -1, ""

            # call function if "callback" is callable
            if callable(self.callback): self.callback(self)
            # if self.block is not None
            if self.block is not None:
                # set convars first
                self["entityid"] = self.entityid
                # execute block
                es.doblock(self.block)

        # if function is "player"
        elif function == "player":
            # get userid of aimed player
            self.playerid = es.getuserid(es.gethandlefromindex(entity))
            if self.playerid:
                self.playerindex = entity
                if self.block is not None: self["classname"] = self.classname
            else: self.playerindex, self.classname = -1, ""

            # call function if "callback" is callable
            if callable(self.callback): self.callback(self)
            # if self.block is not None
            if self.block is not None:
                # set convars first
                self["playerid"] = self.playerid
                self["playerindex"] = self.playerindex
                # execute block
                es.doblock(self.block)

        # if function is "weapon"
        elif function == "weapon":
            # if classname does not starts with "weapon_"
            if not self.classname.startswith(_weapon): self.weaponid, self.classname = -1, ""
            elif not self.classname.startswith(self._entityname): self.weaponid, self.classname = -1, ""
            else:
                self.weaponid = entity
                if self.block is not None: self["classname"] = self.classname

            # call function if "callback" is callable
            if callable(self.callback): self.callback(self)
            # if self.block is not None
            if self.block is not None:
                # set convars first
                self["weaponid"] = self.weaponid
                # execute block
                es.doblock(self.block)

    """ coord function """
    def i_coord(self, index):
        # get coordinates
        self.coordinates = es.getindexprop(index, "CBaseEntity.m_vecOrigin")
        targetname = "_playerview_coord_%s" % self.userid
        # set new targetname
        es.setentityname(index, targetname)
        # remove material
        userid = es.getuserid()
        if userid: es.server.insertcmd("es_xfire %s %s kill " % (userid, targetname))
        # remove player from dict
        del _players[self.userid]

        # set function name
        self.function = "coord"
        # call function with "userid" and "index" parameter
        if callable(self.callback): self.callback(self)
        # if self.block is not None
        if self.block is not None:
            # set convars first
            self["function"] = "coord"
            self["userid"] = self.userid
            self["coordinates"] = self.coordinates
            # execute block
            es.doblock(self.block)


    """ Entity Method """
    def entity(self, entity = ""):
        self._entityname = entity
        # check user, refresh entity list, add object in players dict
        self.i_command()
        # set entity targetname, exec internal server command
        es.server.insertcmd("es_xentsetname %s _playerview_entity_%s;_playerview_cmd %s entity" % (self.userid, self.userid, self.userid))

    """ Player Method """
    def player(self):
        # check user, refresh entity list, add object in players dict
        self.i_command()
        # set entity targetname, exec internal server command
        es.server.insertcmd("es_xentsetname %s _playerview_player_%s;_playerview_cmd %s player" % (self.userid, self.userid, self.userid))

    """ Weapon Method """
    def weapon(self, weapon = ""):
        self._entityname = weapon if weapon.startswith(_weapon) else ""
        # check user, refresh entity list, add object in players dict
        self.i_command()
        # set entity targetname, exec internal server command
        es.server.insertcmd("es_xentsetname %s _playerview_weapon_%s;_playerview_cmd %s weapon" % (self.userid, self.userid, self.userid))

    """ Coord Method """
    def coord(self):
        # check user, refresh entity list, add object in players dict
        self.i_command(False)
        # create dynamic prop, exec internal server command
        es.server.insertcmd(
            "es_xprop_dynamic_create %s props_c17/tv_monitor01_screen.mdl;" % self.userid +
            "es _playerview_cmd %s coord server_var(eventscripts_lastgive)" % self.userid)



def entity(userid, callback, entity = ""):
    # playerviewlib.entity(<userid>, <callback>, [entity])
    PlayerView(userid, callback).entity(entity)

def player(userid, callback):
    # playerviewlib.player(<userid>, <callback>)
    PlayerView(userid, callback).player()

def weapon(userid, callback, weapon = ""):
    # playerviewlib.player(<userid>, <callback>, [weapon])
    PlayerView(userid, callback).weapon(weapon)

def coord(userid, callback):
    # playerviewlib.player(<userid>, <callback>)
    PlayerView(userid, callback).coord()



def es_map_start(event_var):
    # clear players dict
    _players.clear()
# registering es_map_start event
es.addons.registerForEvent(__import__(__name__), "es_map_start", es_map_start)
